--[[----------------------------------------------------------------------------

MuyphotoAPI.lua
Common code to initiate Kuvatilaus.fi API requests

--------------------------------------------------------------------------------

Copyright 2017 Kuvatilaus.fi
All Rights Reserved.

------------------------------------------------------------------------------]]

	-- ZeroBrane debugging
--local LrMobdebug = import 'LrMobdebug'

	-- Lightroom SDK
local LrApplication = import "LrApplication"
local LrBinding = import 'LrBinding'
local LrDate = import 'LrDate'
local LrDialogs = import 'LrDialogs'
local LrErrors = import 'LrErrors'
local LrFunctionContext = import 'LrFunctionContext'
local LrHttp = import 'LrHttp'
local LrMD5 = import 'LrMD5'
local LrPathUtils = import 'LrPathUtils'
local LrView = import 'LrView'

local prefs = import 'LrPrefs'.prefsForPlugin()

local bind = LrView.bind
local share = LrView.share

local logger = import 'LrLogger'( 'MuyphotoLightroomPlugin' )
logger:enable('print')

--============================================================================--

JSON = (loadfile (LrPathUtils.child(_PLUGIN.path, "JSON.lua")))()

--============================================================================--

MuyphotoAPI = {}

local appearsAlive
local endPoint = "https://muyphoto.com/api/"

function MuyphotoAPI.getAPIUrl(action, params)

	url = endPoint .. action

	if params then
		return string.format('%s?%s', url, Utils.makeQueryString(params))
	else
		return url
	end
end

function MuyphotoAPI.makeRequest(action, params, method)

	Utils.logInfo("MuyphotoAPI.makeRequest:", action)

	local url
	if method == 'POST' then
		url = MuyphotoAPI.getAPIUrl(action)
	else
		url = MuyphotoAPI.getAPIUrl(action, params)
	end

	-- Call the URL and wait for response.

	if method == 'POST' then
		queryString = Utils.makeQueryString(params)
		requestHeaders = {
			{ field = "Content-Type", value = "application/x-www-form-urlencoded" },
			{ field = "User-Agent", value = "Muyphoto Lightroom Plugin 2.0" }
		}
		response, headers = LrHttp.post( url, queryString, requestHeaders)
	else
		response, headers = LrHttp.get( url )
	end

	-- If headers.error is set, that means Lightroom had an error.
	if headers and headers.error then
		Utils.logError("checkResponse: Lightroom network error for url: " .. url)
		Utils.logTable(headers, "checkResponse headers")
		return { status = "error", from = "lightroom", code = headers.error.errorCode, description = headers.error.name }
	end

	-- Alternatively, the server could throw back an error.
	-- Only return response if we're sure
	if headers and tonumber(headers.status) > 299 then
		Utils.logError("checkResponse: Server error " .. headers.status .. " for url: " .. url)
		return { status = "error", from = "server", code = headers.status, payload = response }
	else
		if response == nil then
			Utils.logError("checkResponse: Response for " .. url .. " was empty.")
			return { status = "error", from = "server", code = 204, payload = nil }
		end
	end

	response_json = JSON:decode(response)
	response_json.status = "ok"

	return response_json
end

function MuyphotoAPI.authLogin(propertyTable)

	propertyTable.loginStatus = "loggingIn"
	propertyTable.loginError = nil

	username, password = propertyTable.username_text, propertyTable.password_text

	local requestParams = {
		username = username,
		password = password
	}

	Utils.logInfo('authLogin.logging in...')

	local loginResponse = MuyphotoAPI.makeRequest('auth', requestParams, 'POST')

	Utils.logInfo('authLogin.logged in!')

	-- Login was successful
	if loginResponse.auth_ok and loginResponse.status ~= "error" then

		propertyTable.loginStatus = "loggedIn"

		propertyTable.loggedUserFullName = loginResponse.full_name
		propertyTable.loggedUserEmail = loginResponse.email
		propertyTable.authToken = loginResponse.auth_token

		MuyphotoAPI.getEvents( propertyTable )

		propertyTable.LR_cantExportBecause = nil

	-- Login error
	else

		propertyTable.loginStatus = "loggedOut"

		if loginResponse.auth_ok == false then
			propertyTable.loginError = LOC "$$$/Export/Strings/InvalidPassword=Invalid e-mail or password"
		else
			propertyTable.loginError = LOC "$$$/Export/Strings/NetworkError=Could not connect to server"
		end

	end

	Utils.logInfo('propertyTable.loginStatus', propertyTable.loginStatus)
	Utils.logInfo('propertyTable.loginError', propertyTable.loginError)

end

function MuyphotoAPI.authCancelLogin(propertyTable)

	propertyTable.loginStatus = "loggedOut"
	propertyTable.loginError = nil
	propertyTable.authToken = nil

	propertyTable.LR_cantExportBecause = LOC "$$$/Export/ExportDialog/Login/LoginRequiredTitle=You haven't signed in yet"

end

function MuyphotoAPI.authLogout(propertyTable)

	propertyTable.loginStatus = "loggedOut"
	propertyTable.loginError = nil
	propertyTable.authToken = nil

	propertyTable.LR_cantExportBecause = LOC "$$$/Export/ExportDialog/Login/LoginRequiredTitle=You haven't signed in yet"

end

function MuyphotoAPI.getEvents(propertyTable)

	--LrMobdebug.start()

	catalog = LrApplication:activeCatalog()

	local returnValue = catalog:withWriteAccessDo( LOC "$$$/Export/SyncEvents/UndoLabel=Update events", function( context )

		--LrMobdebug.on()

		catalog:assertHasWriteAccess( "getEvents" )

		publishServices = catalog:getPublishServices( _PLUGIN.id )
		local lastYearName = nil
		local lastMonthName = nil
		local yearCollectionSet = nil
		local monthCollectionSet = nil
		for _, publishService in ipairs( publishServices ) do

			if propertyTable == nil then
				serviceSettings = publishService:getPublishSettings()
			else
				serviceSettings = propertyTable
			end

			Utils.logInfo('serviceSettings.authToken:', serviceSettings.authToken)

			if serviceSettings.loginStatus == "loggedIn" and serviceSettings.authToken ~= nil then

				requestParams = {
					auth = serviceSettings.authToken
				}
				loginResponse = MuyphotoAPI.makeRequest('events', requestParams, 'GET')

				if loginResponse.status == "error" then
					LrErrors.throwUserError( LOC "$$$/Export/SyncEvents/Error=Could not get events from server. Please contact administration and try again later." )
					return false
				end

				Utils.logInfo("update event types")

				eventTypes = {}
				for k, v in pairs(loginResponse.event_types) do
					table.insert(eventTypes, { key = k, title = v })
				end

				if propertyTable ~= nil then
					propertyTable.eventTypes = eventTypes
				end

				for _, event in pairs( loginResponse.events ) do

					yearCollectionSetName = string.format("%s", event.year)
					monthCollectionSetName = string.format("%s %s", event.month, Utils.monthNames[event.month])
					collectionName = string.format("%s %s", event.day, event.name, event.id)

					if yearCollectionSetName ~= lastYearName then
						yearCollectionSet = publishService:createPublishedCollectionSet( yearCollectionSetName, nil, true )
						lastYearName = yearCollectionSetName
					end

					if monthCollectionSetName ~= lastMonthName then
						monthCollectionSet = publishService:createPublishedCollectionSet( monthCollectionSetName, yearCollectionSet, true )
						lastMonthName = monthCollectionSetName
					end

					publishedCollection = publishService:createPublishedCollection( collectionName, monthCollectionSet, true )

					if publishedCollection then
						publishedCollection:setRemoteId( event.id )
						publishedCollection:setRemoteUrl( event.link )
						publishedCollection:setCollectionSettings({
							eventDateTime   = event.datetime,
							eventDateDay    = event.day,
							eventDateMonth  = event.month,
							eventDateYear   = event.year,
							eventType       = event.type,
							eventTypeLabel  = event.type_label,
						})
					end
				end
			--else
			--	LrErrors.throwUserError("Kuvaustapahtumien haku ei onnistunut, kirjautumistiedot eivät täsmää. Kirjaudu ensin ulos ja sitten uudelleen sisään.")
			--	return false
			end
		end
	end )
end

function MuyphotoAPI.uploadPhoto( propertyTable, params )

	local remotePhotoId = nil

	local authToken = propertyTable.authToken or params.authToken
	local targetURL = MuyphotoAPI.getAPIUrl('photos', { auth = authToken, event = params.eventId })
	local filePath = assert( params.filePath )
	local fileName = LrPathUtils.leafName( filePath )
	params.fileName = fileName
	
	local mimeChunks = {}
	for argName, argValue in pairs( params ) do
		mimeChunks[ #mimeChunks + 1 ] = { name = argName, value = argValue }
	end
	mimeChunks[ #mimeChunks + 1 ] = { name = 'photo', fileName = fileName, filePath = filePath, contentType = 'application/octet-stream' }

	local response, headers = LrHttp.postMultipart( targetURL, mimeChunks )

	if not response then
		if headers and headers.error then
			LrErrors.throwUserError( formatError( hdrs.error.nativeCode ) )
		end		
	end

	response_json = JSON:decode(response)

	remotePhotoId = response_json.photo_id

	return remotePhotoId

end

function MuyphotoAPI.organizePhotos( propertyTable, eventId, uploadedPhotoIds )

	requestParams = {
		auth = propertyTable.authToken,
		force_analyze = true,
		uploaded_photo_uuids = table.concat(uploadedPhotoIds, ',')
	}
	MuyphotoAPI.makeRequest('events/' .. eventId .. '/organize_photos', requestParams)

end
